# Copyright (c) 2006, Stephen Fisher, Susan Davidson, and Junhyong Kim, 
# University of Pennsylvania.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301 USA.
#
# exBatchScript.py - sample batch script

# This example will create three 1000 leaf sub-trees by randomly
# sampling the million leaf tree ("TREE-0") in the SDSC database. Each
# of the three sub-tree will be exported to a NEXUS file, excluding
# sequence data.

# Load standard definitions
from startup import *

# Specify that we will be connecting to an Oracle server
setDBType("oracle")

# Open a read-only connection to the SDSC Oracle server, via web proxy
openDatabase("", "", "http://www.phylo.org/DatabaseProxy.php")

# Create new query called 'SampleQuery', querying the tree "TREE-0"
query = Query("TREE-0", "SampleQuery")

# Randomly sample the tree.
# 0 = Select All
# 1 = Random Selection
# 2 = Select By Temporal Depth (distributed)
# 3 = Select By Temporal Depth (weighted)
# 4 = Selection by Level (distributed)
# 5 = Selection by Level (weighted)
# 6 = Manual Selection
query.setLeafSelection(1)

# Include 1000 leaves in sampled tree
query.setNumLeaves(1000)

# Do not include sequence data in the output.
# 0 = Select All
# 1 = Random Codon Selection
# 2 = Randon Position Selection
# 3 = Manual Codon Selection
# 4 = Manual Position Selection
# 5 = Select None
query.setSequenceSelection(5)

# Run query three times. Output is saved to NEXUS files called
# sampTree1000_0, sampTree1000_1, sampTree1000_2. The resulting NEXUS
# files will only include the tree blocks as the sequence selection
# was "5" (Select None). Query notes will not be included. These runs
# will not be reproducible as the random seed was not set for each
# run.
runQuery("SampleQuery", "sampTree1000", 0, 3)
