#!/usr/bin/python

# CRIMSON
# Copyright (c) 2006, Stephen Fisher, Susan Davidson, and Junhyong Kim, 
# University of Pennsylvania.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301 USA.
#
# split_nex.py - splits nexus file into 3 files for loading into
#                Oracle using sqlldr
#
# @author  Stephen Fisher
# @version $Id: split_nex.py,v 1.3 2007/06/08 20:09:10 fisher Exp $

# grep -v TREE <file> | grep -v DIM | grep -v MATRIX | grep -v FORMAT | grep -v NEXUS | split_nex.py

import sys
import string

try:
    input=sys.argv[1]
    output=sys.argv[2]
except IndexError:
    print "Usage: split_nex <nexus file> <output>"
    sys.exit(0)

iFile = open(input, "r")
if iFile == None: sys.exit(0)


taxaFile = open(output + ".tax", "w")
if taxaFile == None: sys.exit(0)
charFile = open(output + ".chr", "w")
if charFile == None: sys.exit(0)
crimFile = open(output + ".crm", "w")
if crimFile == None: sys.exit(0)

# flag when in CHARACTER or CRIMSON blocks
charFlag = 0
crimFlag = 0
while 1:
    line = iFile.readline()
    if not line: break
    line = string.lstrip(line)

    # step past the taxa block. we ignore the taxa labels here because
    # we'll propogate the taxa file when we process the character
    # block.
    if line.upper().startswith("BEGIN TAXA"):
        while 1:
            line = iFile.readline()
            if not line:
                print "ERROR: malformed Nexus file"
                sys.exit(0)
            if line.startswith("END"): break
        continue
        
    if line.upper().startswith("BEGIN DATA") or line.upper().startswith("BEGIN CHARACTERS"):
        charFlag = 1
        continue
    if line.upper().startswith("BEGIN CRIMSON"):
        crimFlag = 1
        continue
    if line.upper().startswith("MATRIX"):
        while 1:
            line = iFile.readline()
            if not line:
                print "ERROR: malformed Nexus file"
                sys.exit(0)

            # split line by whitespace (space and/or tabs)
            line = string.lstrip(line)

            if line.startswith(";") or line.startswith("END"):
                charFlag = 0
                crimFlag = 0
                break
            if charFlag:
                cols = line.split()
                # first column is the taxa labels
                taxaFile.write(cols[0] + "\n")
                charFile.write(cols[1] + "\n")
                continue
            if crimFlag:
                crimFile.write(line.split()[1] + "\n")
                continue
        continue
