# CRIMSON
# Copyright (c) 2006, Stephen Fisher, Susan Davidson, and Junhyong Kim, 
# University of Pennsylvania.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301 USA.
#
# startup.py - initialize console
#
# @author  Stephen Fisher
# @version $Id: startup.py,v 1.141 2009/07/25 03:37:09 fisher Exp $


# load standard definitions
from header import *

#----------------------------------------------------------------------------
# General functions
#----------------------------------------------------------------------------

def quit():
	"""quit():
	Quits the application.
	"""
	exit(0)

def isEmpty(val):
	"""isEmpty(val):
	Tests for empty and null string
	"""
	return CrimsonUtils.isEmpty(val)

def execute(cmd):
	"""execute(cmd):
	Executes the command 'cmd'.  If Windows OS then will prepend 'cmd
	/c'. 
	"""
	return CrimsonMain.exec(cmd, 1)

def loadPython(script):
	"""loadPython(script):
	Loads the python script file into the command line. The file
	can not contain function definitions. Function defs should be
	loaded into the command line with: 'from <file> import *'
	"""
	return FileIO.loadPython(script)

def writeString(filename, append, out):
	"""writeString(filename, append, out):
	Writes 'out' to the file specified. If append is true then the
	text is appended to the end of the file. Otherwise the file is
	overwritten.
	"""
	FileIO.writeString(filename, append, out)

def viewAPI():
	"""viewAPI():
	Launch HTML viewer to view Java API documentation.
	"""
	return ViewHTML(java.net.URL("file:documentation/index.html"))

def resetGlobals():
	"""resetGlobals():
	Reset user-definable global variables to their default values.
	"""
	resetCrimsonDefaults()
	GUIUtils.resetGUIDefaults()

def about():
	"""about():
	Prints version information.
	"""
	printMsg("CRIMSON")
	printMsg("Version 2.1 beta")
	printMsg("By Stephen Fisher, Susan Davidson, and Junhyong Kim.")
	printMsg("Copyright (c) 2006, University of Pennsylvania.")
	printMsg("CRIMSON comes with ABSOLUTELY NO WARRANTY.  This is")
	printMsg("free software, and you are welcome to redistribute it")
	printMsg("under the terms of the GNU General Public License.")
	
def time(*args):
	"""time([inMillis]):
	Returns the current time. If inMillis is true then time is
	returned in milliseconds.
	"""
	if len(args) == 1:
		if args[0]: 
			return java.util.Calendar.getInstance().getTimeInMillis()
	return java.util.Calendar.getInstance().time

def help(*args):
	"""Prints help for user-defined functions:
	help(): print list of commands.
	help(<UDF>): print extended documentation for 'UDF'.
	"""
	if len(args) == 0: helpList()
	if len(args) == 1: printMsg(args[0].__doc__)

def helpList():
	printMsg("For help on a specific built-in functions type 'help(<name>)' at the console.")
	printMsg("Example: help(openDatabase)")
	printMsg("  GENERAL FUNCTIONS:")
	printMsg("    quit()")
	printMsg("    execute()")
	printMsg("    isEmpty()")
	printMsg("    loadPython()")
	printMsg("    viewAPI()")
	printMsg("    resetGlobals()")
	printMsg("    about()")
	printMsg("    time()")
	printMsg("    help()")
	printMsg("  CRIMSON FUNCTIONS:")
	printMsg("    setVerbose()")
	printMsg("    setTmpDir()")
	printMsg("    getTmpDir()")
	printMsg("    setUsername()")
	printMsg("    setDatabase()")
	printMsg("    getTree()")
	printMsg("    getPartition()")
	printMsg("    getModel()")
	printMsg("    getQuery()")
	printMsg("    importQuery()")
	printMsg("    exportQuery()")
	printMsg("  CRIMSON FUNCTIONS:")
	printMsg("    openDatabase()")
	printMsg("    closeDatabase()")
	printMsg("    testConnection()")
	printMsg("    execUpdate()")
	printMsg("    execSQL()")
	printMsg("    loadTree()")
	printMsg("    appendTree()")
	printMsg("    removeTmpFiles()")
	printMsg("    exportTree()")
	printMsg("    exportStruct()")
	printMsg("    exportPartition()")
	printMsg("    deleteTree()")
	printMsg("    deletePartition()")
	printMsg("    loadAllTrees()")
	printMsg("    loadAllModels()")
	printMsg("    deleteModel()")
	printMsg("    loadAllQueries()")
	printMsg("    loadQuery()")
	printMsg("    publishQuery()")
	printMsg("    runQuery()")
	printMsg("    runPhylipQuery()")
	printMsg("    runNexusQuery()")
	printMsg("    deleteQuery()")
	printMsg("    deleteQueryFromDatabase()")
	printMsg("  GUI FUNCTIONS:")
	printMsg("    menu()")
	printMsg("    treeManager()")
	printMsg("    modelManager()")
	printMsg("    newModel()")
	printMsg("    editModel()")
	printMsg("    viewModel()")
	printMsg("    queryManager()")
	printMsg("    newQuery()")
	printMsg("    editQuery()")
	printMsg("    viewQuery()")
	printMsg("  USER-DEFINED FUNCTIONS:")
	printMsg("    nex2wal()")
	printMsg("    runWalrus()")

#----------------------------------------------------------------------------
# edu.upenn.crimson functions
#----------------------------------------------------------------------------

def setVerbose(val):
	"""setVerbose(val):
	Changes the feedback verbosity of the application.  The amount of
	detailed feedback is as follows: 2 = lots, 1 = no warnings, 0 = no
	feedback.  This value will persist across instances of the
	application.
	"""
	CrimsonUtils.setVerbose(val)

def setTmpDir(*args):
	"""setTmpDir([val]):
	Changes the default directory used for temporary dataloader files
	(CrimsonUtils.TMP_DIR).
	"""
	if len(args) == 1: CrimsonUtils.setTmpDir(args[0])
	else: CrimsonUtils.setTmpDir(GUIUtils.getFilename(GUIUtils.directoryChooser()))

def getTmpDir():
	"""getTmpDir():
	Retruns the default directory used for temporary dataloader files.
	"""
	return CrimsonUtils.getTmpDir()

def setUsername(val):
	"""setUsername(val):
	Changes the default username used to connect to database.  This is
	also updated every time a valid database connection is made.
	"""
	CrimsonUtils.setUsername(val)

def setDatabase(val):
	"""setDatabase(val):
	Changes the default database used.  This is also updated every time
	a valid database connection is made.
	"""
	CrimsonUtils.setDatabase(val)

def getTree(id):
	"""getTree(id):
	Returns the tree object from the treePool.
	"""
	return ObjectHandles.getTree(id)

def getPartition(id):
	"""getPartition(id):
	Returns the partition object from the tree.
	"""
	return ObjectHandles.getPartition(id)

def getModel(id):
	"""getModel(id):
	Returns the model object from the modelPool.
	"""
	return ObjectHandles.getModel(id)

def getQuery(id):
	"""getQuery(id):
	Returns the query object from the queryPool.
	"""
	return ObjectHandles.getQuery(id)

def importQuery(filename):
	"""importQuery(filename):
	This will load a query from the specified python file.
	"""
	FileIO.loadPython(filename)

def exportQuery(id, path):
	"""exportQuery(id, path):
	This will save the query to a python file that can be reload into
	jython. If the file already exists, it will be overwritten.
	"""
	FileIO.exportQuery(id, path, 1)

#----------------------------------------------------------------------------
# edu.upenn.crimson.db functions
#----------------------------------------------------------------------------

def setDBType(type):
	"""setDBType(type):
	Sets the type of database (Oracle or MySQL).
	"""
	if type.strip() == "": CrimsonUtils.printWarning("Database type not set.")
	if SQL.setDBType(type): printMsg("Using " + type + " database.")

def openDatabase(*args):
	"""openDatabase([username, password, server [, port, database]]):
	Connects to the specified database.  If no args, then a graphical
	dialog will be presented.  When a valid connection is made, the
	default values for username, database, server, and port will be
	updated.
	"""
	if len(args) == 0: GUIUtils.openDatabase()
	elif len(args) == 3: Database.open(args[0], args[1], args[2], 0, "")
	elif len(args) == 5: Database.open(args[0], args[1], args[2], args[3], args[4])
	else: printError("Invalid number of arguments in openDatabase().")

def closeDatabase():
	"""closeDatabase():
	Closes the connection to the database.
	"""
	Database.close()
	
def testConnection():
	"""testConnection():
	Performs a simple SQL query to test the database connection.
	"""
	Database.testConnection()
	
def execUpdate(sql):
	"""execUpdate(sql):
	This will execute SQL commands that update the database (ie CREATE,
	DROP, INSERT, DELETE, etc). This will return 1 if it completes or 0
	on error. COMMIT will automatically be run after the execution of
	the sql statement.
	"""
	return Database.execUpdate(sql)
	
def execSQL(sql):
	"""execSQL(sql):
	This will execute SQL queries of the database and will return a
	ArrayList containing HashMaps, with each HashMap containing one row
	of data. The keys of the HashMap are the column names. This can be
	used, for example, to run Oracle 'SELECT' statements.
	"""
	if Database.isOpen(): 
		return Database.sqlToArray(sql)
	else:
		printError("Must open a database before running a query.")

def loadTree(*args):
	"""loadTree(filename, treeID [, partitionID]):
	This will load the NEXUS file 'filename'.  
	"""
	if len(args) == 1:
		printError("Invalid number of arguments in loadTree().")
		print help(loadTree)
		return
	if len(args) == 2: tree = NexusFile.load(args[0], args[1], args[1] + "_p1")
	else: tree = NexusFile.load(args[0], args[1], args[2])
	# GC is run in clearStructure() but that isn't always sufficient
	CrimsonMain.runGC()
	return tree

def appendTree(filename, treeID, partitionID):
	"""appendTree(filename, treeID, partitionID):
	This will load the NEXUS file 'filename'.
	"""
	NexusFile.append(filename, treeID, partitionID)

def removeTmpFiles(treeID):
	"""removeTmpFiles(treeID):
	This will remove any files stored in CrimsonUtils.TMP_DIR.  
	"""
	if isEmpty(treeID): treeID = TreeUtils.treeSelector()
	NexusFile.removeTmpFiles(treeID)

def exportTree(treeID):
	"""exportTree(treeID):
	This will save the tree and all partition data as a NEXUS file.
	"""
	_exportTree(treeID, None, 0)

def exportStruct(treeID):
	"""exportStruct(treeID):
	This will save the tree structure as a NEXUS file.
	"""
	_exportTree(treeID, None, 1)

def exportPartition(treeID, partitionID):
	"""exportPartition(treeID, partitionID):
	This will save the selected partition to a NEXUS file.
	"""
	_exportTree(treeID, partitionID, 0)

def _exportTree(treeID, partitionID, onlyStruct):
	"""_exportTree(treeID, partitionID, onlyStruct):
	This is an internal function that does the work for exportTree(),
	exportStruct(), and exportPartition().
	"""
	# create a temporary query with the necessary tree info
	query = Query(treeID)
	if not partitionID == None: query.addPartition(partitionID)
	query.setOnlyStruct(onlyStruct)

	# this will prompt the user for the output info and then run the query
	runQuery(query.getID())

	# remove the query now that we're finished with it
	ObjectHandles.deleteQuery(query.getID())
	

def deleteTree(*args):
	"""deleteTree(id [, delQueries]):
	Deletes the tree from the database. If any queries reference the
	tree and 'delQueries' is true, then the queries will also be
	deleted.  If 'delQueries' is false (default value) and related
	queries are found, then the tree will not be deleted.
	"""
	if len(args) == 1: return Trees.delete(args[0], 0)
	else: return Trees.delete(args[0], args[1])

def deletePartition(id):
	"""deletePartition(id):
	Deletes the partition from the database.
	"""
	return Partitions.delete(id)

def loadAllTrees():
	"""loadAllTrees():
	This will load all of the trees in the current database.
	"""
	Trees.loadAll()
	Partitions.loadAll()

def loadAllModels():
	"""loadAllModels():
	This will load all of the models in the current database.
	"""
	Models.loadAll()

def deleteModel(id):
	"""deleteModel(id):
	Moves a model from the database into the recycle bin.  The model can
	thus be restored or permanently deleted (when the Oracle recycle bin is
	purged).
	"""
	return Models.delete(id)

def loadAllQueries():
	"""loadAllQueries():
	This will load all of the queries in the current database. If a
	query already exists with the same id, then the new query id
	will be changed so as to not conflict with the existing query.
	"""
	Queries.load("")

def loadQuery(id):
	"""loadQuery(id):
	This will load the specified query from database.
	"""
	Queries.load(id)

def publishQuery(id):
	"""publishQuery(id):
	This will add the specified query to the database if it doesn't
	already exist in the database.  If it already exists then it will
	update the database entry.
	"""
	Queries.publish(getQuery(id))

def runQuery(*args):
	"""runQuery(id [, output, incSequence, runs]):
	This will run the specified query, saving the output in the
	specified NEXUS file. Runs is the number of times the query
	will be run. If the query.seed() is set, then runs can't be
	more than 1. If only the query ID is given, then a dialog will
	be presented to get the remaining information. The output will
	be a NEXUS file that includes the Crimson and tree
	blocks. Query notes will not be included.
	"""
	if len(args) == 1: QueryDialog(args[0])
	else: Queries.run(args[0], args[1], args[2], args[3])

def runPhylipQuery(id, output, incTree):
	"""runQuery(id, output, incTree):
	This will run the specified query, saving the output in the
	specified phylip file. The generated tree will be returned or
	Null on error. There are multiple options dictated by incTree:
	  incTree = 0 -> no tree will be output
	  incTree = 1 -> no tree will be output
	  incTree = 2 -> tree ouptut to second NEXUS file ("<output>.tree")
	  incTree = 3 -> rooted tree ouptut to a newick file ("<output>.newick")
	  incTree = 4 -> unrooted tree ouptut to a newick file ("<output>.newick")
	"""
	return Queries.runPhylip(id, output, incTree)

def runNexusQuery(id, output, incSequence, incCrimson, incTree, incNotes):
	"""runNexusQuery(id, output, incSequence, incCrimson, incTree, incNotes):
	This will perform a tree query. If incCrimson is false then
	the CRIMSON block will not be included. If incNotes is true then the
	Query.notes will be appended to the end of the NEXUS file. The
	notes can, for example, contain a PAUP block. It is assumed
	that the notes are properly formatted for the NEXUS file. If
	Query.onlyStruct is true then incCrimson and incTree will be
	ignored. The generated tree will be returned or Null on error.
	There are multiple options dictated by incTree:
	  incTree = 0 -> no tree will be output
	  incTree = 1 -> tree is included in NEXUS file
	  incTree = 2 -> tree ouptut to second NEXUS file ("<output>.tree")
	  incTree = 3 -> rooted tree ouptut to a newick file ("<output>.newick")
	  incTree = 4 -> unrooted tree ouptut to a newick file ("<output>.newick")
	"""
	return Queries.run(id, output, incSequence, incCrimson, incTree, incNotes)

def deleteQuery(id, fromDatabase):
	"""deleteQuery(id, fromDatabase):
	This will remove the specified query from the the queryPool. If
	'fromDatabase' is true (= 1), then it will also remove the query
	from the database.
	"""
	if fromDatabase: deleteQueryFromDatabase(id)
	else: ObjectHandles.deleteQuery(id)

def deleteQueryFromDatabase(id):
	"""deleteQueryFromDatabase(id):
	This will remove the specified query from the database and the
	queryPool.
	"""
	Queries.delete(id)
	ObjectHandles.deleteQuery(id)


#----------------------------------------------------------------------------
# edu.upenn.crimson.io functions
#----------------------------------------------------------------------------

#----------------------------------------------------------------------------
# edu.upenn.crimson.gui functions
#----------------------------------------------------------------------------

def menu():
	"""menu():
	States the application's GUI. 
	"""
	if not NOX: Root.show()

def treeManager():
	"""treeManager():
	Displays the tree manager.
	"""
	if not Database.isOpen(): printError("Must open a database before opening the tree manager.");
	else:	TreeManager.show()

def modelManager():
	"""modelManager():
	Displays the model manager.
	"""
	if not Database.isOpen(): printError("Must open a database before opening the model manager.");
	else:	ModelManager.show()

def newModel():
	"""newModel():
	This will create a new model object.  The model object will then be
	opened in a ModelEditor panel.
	"""
	# get model ID
	id = ModelUtils.getModelID("")
	
	# if id is null, then cancelled getModelID() so quit
	if id == None: return

	# create model
	model = Model(id)

	# edit model
	ModelEditor(model)
	return model

def editModel(id):
	"""editModel(id):
	This will open the model object in a ModelEditor panel.
	"""
	if ObjectHandles.containsModel(id): ModelEditor(getModel(id))
	else: printError("Model not found")

def viewModel(*args):
	"""viewModel([id]):
	This will display the model's parameter values.
	"""
	if len(args) == 0: ModelUtils.viewModel("")
	else: ModelUtils.viewModel(args[0])

def queryManager():
	"""queryManager():
	Displays the query manager.
	"""
	if not Database.isOpen(): printError("Must open a database before opening the query manager.");
	else:	QueryManager.show()

def newQuery(treeID):
	"""newQuery(treeID):
	This will create a new query object that is set to edit the
	specified tree.  The query object will then be opened in a
	QueryEditor panel.
	"""
	# get query ID
	queryID = QueryUtils.getQueryID(treeID + "_q")
	
	# if not a valid ID, then cancelled getQueryID() so quit
	if queryID == None: return

	# create query
	query = Query(treeID, queryID)

	# edit query
	QueryEditor(query)
	return query

def editQuery(id):
	"""editQuery(id):
	This will open the query object in a QueryEditor panel.
	"""
	if ObjectHandles.containsQuery(id): QueryEditor(getQuery(id))
	else: printError("Query not found")

def viewQuery(*args):
	"""viewQuery([id]):
	This will display the query's parameter values.
	"""
	if len(args) == 0: QueryUtils.viewQuery("")
	else: QueryUtils.viewQuery(args[0])

#----------------------------------------------------------------------------
# load User-Defined Functions
#----------------------------------------------------------------------------

def tree2wal(*args):
	"""tree2wal([id [, outFile, view, min]]):
	This will display a tree in the Walrus 3D viewer.
	"""
	if len(args) == 0 or isEmpty(args[0]): id = TreeUtils.treeSelector()
	else: id = args[0]
	# if still blank, the quit
	if isEmpty(id): return
	
	tree = getTree(id)
	if tree == None:
		printError("Invalid tree ID: " + id)
		return
	if not tree.buildTree():
		printError("Invalid tree: " + id)
		return
	
	if len(args) > 1: outFile = args[1]
	else: outFile = ""
	if isEmpty(outFile):
		# get output file
		outFile = getTmpDir() + "/_tmpOutput.wal"

		#outFile = GUIUtils.saveFileChooser("Select Output File", outFile, None)
		# convert outFile from java.io.File to String
		#outFile = GUIUtils.getFilename(outFile)
		# make sure filename is valid
		#if isEmpty(outFile): return
	
	if len(args) > 1 and args[3]: nex2walrus.export(tree, outFile, 0, 0, 0, 0)
	else: nex2walrus.export(tree, outFile, 1, 1, 1, 0)
	printMsg("Conversion completed.")

	# if 'view' is true, then open output in new viewer, if not
	# included then assume it's true.  runWalrus() does the same
	# thing as GUIUtils.launchWalrus()
	if len(args) > 1:
		if args[2]: runWalrus(outFile)
	else: runWalrus(outFile)

	#if args[2]: GUIUtils.launchWalrus(outFile)
	#else: GUIUtils.launchWalrus(outFile)

def nex2wal(*args):
	"""nex2wal([inFile [, outFile [, view [, min]]]]):
	This will convert a nexus file to a walrus format for viewing in
	Walrus.  If the input or output file names are empty, then a
	graphical dialog will be presented to query the user for the file
	names.  If 'view' is true, then after converting the file, it will
	be displayed in a new Walrus viewer, if not present then it is
	assumed to be true.  If 'min' is true, then the output file will
	not contain any extra display info, such as species IDs or stem
	lengths.  This will produce a significantly smaller file that will
	only allow for the display of the tree structure.
	"""
	if len(args) > 0: inFile = args[0]
	else: inFile = ""
	if isEmpty(inFile):
		# get input file
		inFile = GUIUtils.openFileChooser("Select NEXUS File", None)
		# convert inFile from java.io.File to String
		inFile = GUIUtils.getFilename(inFile)
		# make sure filename is valid
		if isEmpty(inFile): return

	if len(args) > 1: outFile = args[1]
	else: outFile = ""
	if isEmpty(outFile):
		# get output file
		outFile = inFile + ".wal"
		outFile = GUIUtils.saveFileChooser("Select Output File", outFile, None)
		# convert outFile from java.io.File to String
		outFile = GUIUtils.getFilename(outFile)
		# make sure filename is valid
		if isEmpty(outFile): return

	tree = NexusFile.loadNewick(inFile)
	# tree = nex2walrus.loadTree(inFile)
	if len(args) > 3 and args[3]: nex2walrus.export(tree, outFile, 0, 0, 0, 0)
	else: nex2walrus.export(tree, outFile, 1, 1, 1, 0)
	printMsg("Conversion completed.")

	# if 'view' is true, then open output in new viewer, if not included
	# then assume it's true.
	if len(args) > 2:
		if args[2]: runWalrus(outFile)
	else: runWalrus(outFile)

def runWalrus(*args):
	"""runWalrus([file]):
	This will display the file using the Walrus 3D viewer.  This script
	is comparable to GUIUtils.launchWalrus().
	"""
	# if WALRUS not loaded, then quit
	if not WALRUS: return

	w = walrus.H3Main()
	if len(args) == 0:
		# get input file
		file = GUIUtils.openFileChooser("Select Walrus Input File", None)
		# make sure file is valid
		if file == None: return
		w.userLoadFile(file)
	else:
		w.userLoadFile(args[0])
	w.handleStartRenderingRequest()

def connectCIPRES():
	"""connectCIPRES():
	Open a read-only connection to the SDSC Oracle server, using a
	web proxy.
	"""
	setDBType("oracle")
	openDatabase("", "", "http://www.phylo.org/DatabaseProxy.php")

#----------------------------------------------------------------------------
# final initializations 
#----------------------------------------------------------------------------

# Launch GUI
if not (ISBATCH or NOX): menu()
